@@ -1,26 +1,32 @@
 use std::borrow::Cow;
 use std::fmt::{self, Debug, Formatter};
-use std::str::FromStr;
+use std::ops::Deref;
 use std::sync::Arc;
 
 use az::SaturatingAs;
-use ecow::EcoString;
-use rustybuzz::{BufferFlags, ShapePlan, UnicodeBuffer};
+use rustybuzz::{BufferFlags, Feature, ShapePlan, UnicodeBuffer};
 use ttf_parser::Tag;
+use ttf_parser::gsub::SubstitutionSubtable;
+use typst_library::World;
 use typst_library::engine::Engine;
 use typst_library::foundations::{Smart, StyleChain};
-use typst_library::layout::{Abs, Dir, Em, Frame, FrameItem, Point, Size};
+use typst_library::layout::{Abs, Dir, Em, Frame, FrameItem, Point, Rel, Size};
+use typst_library::model::{JustificationLimits, ParElem};
 use typst_library::text::{
-    families, features, is_default_ignorable, variant, Font, FontFamily, FontVariant,
-    Glyph, Lang, Region, TextEdgeBounds, TextElem, TextItem,
+    Font, FontFamily, FontVariant, Glyph, Lang, Region, ShiftSettings, TextEdgeBounds,
+    TextElem, TextItem, families, features, is_default_ignorable, language, variant,
 };
-use typst_library::World;
 use typst_utils::SliceExt;
 use unicode_bidi::{BidiInfo, Level as BidiLevel};
 use unicode_script::{Script, UnicodeScript};
 
-use super::{decorate, Item, Range, SpanMapper};
-use crate::modifiers::{FrameModifiers, FrameModify};
+use super::{Item, Range, SpanMapper, decorate};
+use crate::modifiers::FrameModifyText;
+
+const SHY: char = '\u{ad}';
+const SHY_STR: &str = "\u{ad}";
+const HYPHEN: char = '-';
+const HYPHEN_STR: &str = "-";
 
 /// The result of shaping text.
 ///
@@ -43,12 +49,81 @@ pub struct ShapedText<'a> {
     pub styles: StyleChain<'a>,
     /// The font variant.
     pub variant: FontVariant,
-    /// The font size.
-    pub size: Abs,
-    /// The width of the text's bounding box.
-    pub width: Abs,
     /// The shaped glyphs.
-    pub glyphs: Cow<'a, [ShapedGlyph]>,
+    pub glyphs: Glyphs<'a>,
+}
+
+/// A copy-on-write collection of glyphs.
+///
+/// At the edges of the collection, there can be _trimmed_ glyphs for end-of-line
+/// whitespace that should not affect layout, but should be emitted into the PDF
+/// to keep things accessible.
+///
+/// These glyphs are not visible when deref-ing the `Glyphs` to a slice as they
+/// should be ignored in almost all circumstances. They are only visible when
+/// explicitly accessing [`all`](Self::all) glyphs.
+#[derive(Clone)]
+pub struct Glyphs<'a> {
+    /// All glyphs, including the trimmed ones.
+    inner: Cow<'a, [ShapedGlyph]>,
+    /// The range of untrimmed glyphs.
+    kept: Range,
+}
+
+impl<'a> Glyphs<'a> {
+    /// Create a borrowed glyph collection from an existing slice.
+    ///
+    /// This happens after reshaping.
+    pub fn from_slice(glyphs: &'a [ShapedGlyph]) -> Self {
+        Self {
+            inner: Cow::Borrowed(glyphs),
+            kept: 0..glyphs.len(),
+        }
+    }
+
+    /// Create an owned glyph collection from an vector.
+    ///
+    /// This happens after initial shaping.
+    pub fn from_vec(glyphs: Vec<ShapedGlyph>) -> Self {
+        let len = glyphs.len();
+        Self { inner: Cow::Owned(glyphs), kept: 0..len }
+    }
+
+    /// Clone the internal glyph data to make it modifiable. Should be avoided
+    /// if possible on potentially borrowed glyphs as it can be expensive
+    /// (benchmarks have shown ~10% slowdown on a text-heavy document if no
+    /// borrowing is used).
+    pub fn to_mut(&mut self) -> &mut [ShapedGlyph] {
+        &mut self.inner.to_mut()[self.kept.clone()]
+    }
+
+    /// Trims glyphs that satisfy the given condition from the start and end.
+    ///
+    /// Those glyphs will not be visible during normal operation anymore, only
+    /// through [`all`](Self::all).
+    pub fn trim(&mut self, f: impl FnMut(&ShapedGlyph) -> bool + Copy) {
+        let (start, end) = self.inner.split_prefix_suffix(f);
+        self.kept = start..end;
+    }
+
+    /// Accesses all glyphs, not just the untrimmed ones.
+    pub fn all(&self) -> &[ShapedGlyph] {
+        self.inner.as_ref()
+    }
+
+    /// Whether this contains neither kept nor trimmed glyphs.
+    pub fn is_fully_empty(&self) -> bool {
+        self.inner.is_empty()
+    }
+}
+
+impl<'a> Deref for Glyphs<'a> {
+    type Target = [ShapedGlyph];
+
+    /// Returns only the kept (untrimmed) glyphs.
+    fn deref(&self) -> &Self::Target {
+        &self.inner[self.kept.clone()]
+    }
 }
 
 /// A single glyph resulting from shaping.
@@ -64,6 +139,8 @@ pub struct ShapedGlyph {
     pub x_offset: Em,
     /// The vertical offset of the glyph.
     pub y_offset: Em,
+    /// The font size for the glyph.
+    pub size: Abs,
     /// The adjustability of the glyph.
     pub adjustability: Adjustability,
     /// The byte range of this glyph's cluster in the full inline layout. A
@@ -87,7 +164,7 @@ pub struct ShapedGlyph {
     pub script: Script,
 }
 
-#[derive(Debug, Clone, Default)]
+#[derive(Debug, Default, Clone)]
 pub struct Adjustability {
     /// The left and right stretchability
     pub stretchability: (Em, Em),
@@ -145,13 +222,43 @@ impl ShapedGlyph {
             || self.c.is_ascii_digit()
     }
 
-    pub fn base_adjustability(&self, style: CjkPunctStyle) -> Adjustability {
+    /// The amount by which the glyph's advance width is allowed to be shrunk or
+    /// stretched to improve justification.
+    pub fn base_adjustability(
+        &self,
+        style: CjkPunctStyle,
+        limits: &JustificationLimits,
+        font_size: Abs,
+        stretchable: bool,
+    ) -> Adjustability {
         let width = self.x_advance;
+
+        // Do not ever shrink away more than three quarters of the glyph. As the
+        // width approaches zero, justification gets increasingly expensive and
+        // if negative it may not terminate.
+        let limited = |v: Em| v.min(width * 0.75);
+
         if self.is_space() {
+            // To a space, both the spacing and tracking limits apply, just like
+            // `text.tracking` and `text.spacing` both apply to spaces.
+            let max = limits.spacing().max + limits.tracking().max;
+            let min = limits.spacing().min + limits.tracking().min;
             Adjustability {
-                // The number for spaces is from Knuth-Plass' paper
-                stretchability: (Em::zero(), width / 2.0),
-                shrinkability: (Em::zero(), width / 3.0),
+                stretchability: (
+                    Em::zero(),
+                    (max - Rel::one())
+                        .map(|length| Em::from_length(length, font_size))
+                        .relative_to(width)
+                        .max(Em::zero()),
+                ),
+                shrinkability: (
+                    Em::zero(),
+                    limited(
+                        (Rel::one() - min)
+                            .map(|length| Em::from_length(length, font_size))
+                            .relative_to(width),
+                    ),
+                ),
             }
         } else if self.is_cjk_left_aligned_punctuation(style) {
             Adjustability {
@@ -168,6 +275,17 @@ impl ShapedGlyph {
                 stretchability: (Em::zero(), Em::zero()),
                 shrinkability: (width / 4.0, width / 4.0),
             }
+        } else if stretchable {
+            Adjustability {
+                stretchability: (
+                    Em::zero(),
+                    Em::from_length(limits.tracking().max, font_size).max(Em::zero()),
+                ),
+                shrinkability: (
+                    Em::zero(),
+                    limited(Em::from_length(-limits.tracking().min, font_size)),
+                ),
+            }
         } else {
             Adjustability::default()
         }
@@ -197,14 +315,6 @@ impl ShapedGlyph {
     }
 }
 
-/// A side you can go toward.
-enum Side {
-    /// To the left-hand side.
-    Left,
-    /// To the right-hand side.
-    Right,
-}
-
 impl<'a> ShapedText<'a> {
     /// Build the shaped text's frame.
     ///
@@ -218,20 +328,24 @@ impl<'a> ShapedText<'a> {
         extra_justification: Abs,
     ) -> Frame {
         let (top, bottom) = self.measure(engine);
-        let size = Size::new(self.width, top + bottom);
+        let size = Size::new(self.width(), top + bottom);
 
         let mut offset = Abs::zero();
         let mut frame = Frame::soft(size);
         frame.set_baseline(top);
 
-        let shift = TextElem::baseline_in(self.styles);
-        let decos = TextElem::deco_in(self.styles);
-        let fill = TextElem::fill_in(self.styles);
-        let stroke = TextElem::stroke_in(self.styles);
-        let span_offset = TextElem::span_offset_in(self.styles);
+        let size = self.styles.resolve(TextElem::size);
+        let shift = self.styles.resolve(TextElem::baseline);
+        let decos = self.styles.get_cloned(TextElem::deco);
+        let fill = self.styles.get_ref(TextElem::fill);
+        let stroke = self.styles.resolve(TextElem::stroke);
+        let span_offset = self.styles.get(TextElem::span_offset);
 
-        for ((font, y_offset), group) in
-            self.glyphs.as_ref().group_by_key(|g| (g.font.clone(), g.y_offset))
+        let mut i = 0;
+        for ((font, y_offset, glyph_size), group) in self
+            .glyphs
+            .all()
+            .group_by_key(|g| (g.font.clone(), g.y_offset, g.size))
         {
             let mut range = group[0].range.clone();
             for glyph in group {
@@ -239,31 +353,47 @@ impl<'a> ShapedText<'a> {
                 range.end = range.end.max(glyph.range.end);
             }
 
-            let pos = Point::new(offset, top + shift - y_offset.at(self.size));
+            let pos = Point::new(offset, top + shift - y_offset.at(size));
             let glyphs: Vec<Glyph> = group
                 .iter()
                 .map(|shaped: &ShapedGlyph| {
-                    let adjustability_left = if justification_ratio < 0.0 {
-                        shaped.shrinkability().0
-                    } else {
-                        shaped.stretchability().0
-                    };
-                    let adjustability_right = if justification_ratio < 0.0 {
-                        shaped.shrinkability().1
-                    } else {
-                        shaped.stretchability().1
-                    };
+                    // Whether the glyph is _not_ trimmed end-of-line
+                    // whitespace. Trimmed whitespace has its advance width and
+                    // offset zeroed out and is not taken into account for
+                    // justification.
+                    let kept = self.glyphs.kept.contains(&i);
 
-                    let justification_left = adjustability_left * justification_ratio;
-                    let mut justification_right =
-                        adjustability_right * justification_ratio;
-                    if shaped.is_justifiable() {
-                        justification_right +=
-                            Em::from_length(extra_justification, self.size)
-                    }
+                    let (x_advance, x_offset) = if kept {
+                        let adjustability_left = if justification_ratio < 0.0 {
+                            shaped.shrinkability().0
+                        } else {
+                            shaped.stretchability().0
+                        };
+                        let adjustability_right = if justification_ratio < 0.0 {
+                            shaped.shrinkability().1
+                        } else {
+                            shaped.stretchability().1
+                        };
 
-                    frame.size_mut().x += justification_left.at(self.size)
-                        + justification_right.at(self.size);
+                        let justification_left = adjustability_left * justification_ratio;
+                        let mut justification_right =
+                            adjustability_right * justification_ratio;
+                        if shaped.is_justifiable() {
+                            justification_right +=
+                                Em::from_abs(extra_justification, glyph_size)
+                        }
+
+                        frame.size_mut().x += justification_left.at(glyph_size)
+                            + justification_right.at(glyph_size);
+
+                        (
+                            shaped.x_advance + justification_left + justification_right,
+                            shaped.x_offset + justification_left,
+                        )
+                    } else {
+                        (Em::zero(), Em::zero())
+                    };
+                    i += 1;
 
                     // We may not be able to reach the offset completely if
                     // it exceeds u16, but better to have a roughly correct
@@ -291,10 +421,10 @@ impl<'a> ShapedText<'a> {
                     // A+B+C+D: Glyph's x_advance
                     Glyph {
                         id: shaped.glyph_id,
-                        x_advance: shaped.x_advance
-                            + justification_left
-                            + justification_right,
-                        x_offset: shaped.x_offset + justification_left,
+                        x_advance,
+                        x_offset,
+                        y_advance: Em::zero(),
+                        y_offset: Em::zero(),
                         range: (shaped.range.start - range.start).saturating_as()
                             ..(shaped.range.end - range.start).saturating_as(),
                         span,
@@ -304,7 +434,7 @@ impl<'a> ShapedText<'a> {
 
             let item = TextItem {
                 font,
-                size: self.size,
+                size: glyph_size,
                 lang: self.lang,
                 region: self.region,
                 fill: fill.clone(),
@@ -327,26 +457,33 @@ impl<'a> ShapedText<'a> {
             offset += width;
         }
 
-        frame.modify(&FrameModifiers::get_in(self.styles));
+        frame.modify_text(self.styles);
         frame
     }
 
+    /// Computes the width of a run of glyphs relative to the font size,
+    /// accounting for their individual scaling factors and other font metrics.
+    pub fn width(&self) -> Abs {
+        self.glyphs.iter().map(|g| g.x_advance.at(g.size)).sum()
+    }
+
     /// Measure the top and bottom extent of this text.
     pub fn measure(&self, engine: &Engine) -> (Abs, Abs) {
         let mut top = Abs::zero();
         let mut bottom = Abs::zero();
 
-        let top_edge = TextElem::top_edge_in(self.styles);
-        let bottom_edge = TextElem::bottom_edge_in(self.styles);
+        let size = self.styles.resolve(TextElem::size);
+        let top_edge = self.styles.get(TextElem::top_edge);
+        let bottom_edge = self.styles.get(TextElem::bottom_edge);
 
         // Expand top and bottom by reading the font's vertical metrics.
         let mut expand = |font: &Font, bounds: TextEdgeBounds| {
-            let (t, b) = font.edges(top_edge, bottom_edge, self.size, bounds);
+            let (t, b) = font.edges(top_edge, bottom_edge, size, bounds);
             top.set_max(t);
             bottom.set_max(b);
         };
 
-        if self.glyphs.is_empty() {
+        if self.glyphs.is_fully_empty() {
             // When there are no glyphs, we just use the vertical metrics of the
             // first available font.
             let world = engine.world;
@@ -388,18 +525,16 @@ impl<'a> ShapedText<'a> {
     pub fn stretchability(&self) -> Abs {
         self.glyphs
             .iter()
-            .map(|g| g.stretchability().0 + g.stretchability().1)
-            .sum::<Em>()
-            .at(self.size)
+            .map(|g| (g.stretchability().0 + g.stretchability().1).at(g.size))
+            .sum()
     }
 
     /// The shrinkability of the text
     pub fn shrinkability(&self) -> Abs {
         self.glyphs
             .iter()
-            .map(|g| g.shrinkability().0 + g.shrinkability().1)
-            .sum::<Em>()
-            .at(self.size)
+            .map(|g| (g.shrinkability().0 + g.shrinkability().1).at(g.size))
+            .sum()
     }
 
     /// Reshape a range of the shaped text, reusing information from this
@@ -418,10 +553,8 @@ impl<'a> ShapedText<'a> {
                 lang: self.lang,
                 region: self.region,
                 styles: self.styles,
-                size: self.size,
                 variant: self.variant,
-                width: glyphs.iter().map(|g| g.x_advance).sum::<Em>().at(self.size),
-                glyphs: Cow::Borrowed(glyphs),
+                glyphs: Glyphs::from_slice(glyphs),
             }
         } else {
             shape(
@@ -438,35 +571,30 @@ impl<'a> ShapedText<'a> {
 
     /// Derive an empty text run with the same properties as this one.
     pub fn empty(&self) -> Self {
-        Self {
-            text: "",
-            width: Abs::zero(),
-            glyphs: Cow::Borrowed(&[]),
-            ..*self
-        }
+        Self { text: "", glyphs: Glyphs::from_slice(&[]), ..*self }
     }
 
-    /// Push a hyphen to end of the text.
-    pub fn push_hyphen(&mut self, engine: &Engine, fallback: bool) {
-        self.insert_hyphen(engine, fallback, Side::Right)
-    }
-
-    /// Prepend a hyphen to start of the text.
-    pub fn prepend_hyphen(&mut self, engine: &Engine, fallback: bool) {
-        self.insert_hyphen(engine, fallback, Side::Left)
-    }
-
-    fn insert_hyphen(&mut self, engine: &Engine, fallback: bool, side: Side) {
+    /// Creates shaped text containing a hyphen.
+    ///
+    /// If `soft` is true, the item will map to plain text as a soft hyphen.
+    /// Otherwise, it will map to a normal hyphen.
+    pub fn hyphen(
+        engine: &Engine,
+        fallback: bool,
+        base: &ShapedText<'a>,
+        pos: usize,
+        soft: bool,
+    ) -> Option<Self> {
         let world = engine.world;
         let book = world.book();
         let fallback_func = if fallback {
-            Some(|| book.select_fallback(None, self.variant, "-"))
+            Some(|| book.select_fallback(None, base.variant, "-"))
         } else {
             None
         };
-        let mut chain = families(self.styles)
-            .filter(|family| family.covers().map_or(true, |c| c.is_match("-")))
-            .map(|family| book.select(family.as_str(), self.variant))
+        let mut chain = families(base.styles)
+            .filter(|family| family.covers().is_none_or(|c| c.is_match("-")))
+            .map(|family| book.select(family.as_str(), base.variant))
             .chain(fallback_func.iter().map(|f| f()))
             .flatten();
 
@@ -475,35 +603,33 @@ impl<'a> ShapedText<'a> {
             let ttf = font.ttf();
             let glyph_id = ttf.glyph_index('-')?;
             let x_advance = font.to_em(ttf.glyph_hor_advance(glyph_id)?);
-            let range = match side {
-                Side::Left => self.glyphs.first().map(|g| g.range.start..g.range.start),
-                Side::Right => self.glyphs.last().map(|g| g.range.end..g.range.end),
-            }
-            // In the unlikely chance that we hyphenate after an empty line,
-            // ensure that the glyph range still falls after self.base so
-            // that subtracting either of the endpoints by self.base doesn't
-            // underflow. See <https://github.com/typst/typst/issues/2283>.
-            .unwrap_or_else(|| self.base..self.base);
-            self.width += x_advance.at(self.size);
-            let glyph = ShapedGlyph {
-                font,
-                glyph_id: glyph_id.0,
-                x_advance,
-                x_offset: Em::zero(),
-                y_offset: Em::zero(),
-                adjustability: Adjustability::default(),
-                range,
-                safe_to_break: true,
-                c: '-',
-                is_justifiable: false,
-                script: Script::Common,
-            };
-            match side {
-                Side::Left => self.glyphs.to_mut().insert(0, glyph),
-                Side::Right => self.glyphs.to_mut().push(glyph),
-            }
-            Some(())
-        });
+            let size = base.styles.resolve(TextElem::size);
+            let (c, text) = if soft { (SHY, SHY_STR) } else { (HYPHEN, HYPHEN_STR) };
+
+            Some(ShapedText {
+                base: pos,
+                text,
+                dir: base.dir,
+                lang: base.lang,
+                region: base.region,
+                styles: base.styles,
+                variant: base.variant,
+                glyphs: Glyphs::from_vec(vec![ShapedGlyph {
+                    font,
+                    glyph_id: glyph_id.0,
+                    x_advance,
+                    x_offset: Em::zero(),
+                    y_offset: Em::zero(),
+                    size,
+                    adjustability: Adjustability::default(),
+                    range: pos..pos + text.len(),
+                    safe_to_break: true,
+                    c,
+                    is_justifiable: false,
+                    script: Script::Common,
+                }]),
+            })
+        })
     }
 
     /// Find the subslice of glyphs that represent the given text range if both
@@ -535,11 +661,7 @@ impl<'a> ShapedText<'a> {
         // Find any glyph with the text index.
         let found = self.glyphs.binary_search_by(|g: &ShapedGlyph| {
             let ordering = g.range.start.cmp(&text_index);
-            if ltr {
-                ordering
-            } else {
-                ordering.reverse()
-            }
+            if ltr { ordering } else { ordering.reverse() }
         });
 
         let mut idx = match found {
@@ -570,7 +692,7 @@ impl<'a> ShapedText<'a> {
         // for the next line.
         let dec = if ltr { usize::checked_sub } else { usize::checked_add };
         while let Some(next) = dec(idx, 1) {
-            if self.glyphs.get(next).map_or(true, |g| g.range.start != text_index) {
+            if self.glyphs.get(next).is_none_or(|g| g.range.start != text_index) {
                 break;
             }
             idx = next;
@@ -599,9 +721,9 @@ pub fn shape_range<'a>(
     range: Range,
     styles: StyleChain<'a>,
 ) {
-    let script = TextElem::script_in(styles);
-    let lang = TextElem::lang_in(styles);
-    let region = TextElem::region_in(styles);
+    let script = styles.get(TextElem::script);
+    let lang = styles.get(TextElem::lang);
+    let region = styles.get(TextElem::region);
     let mut process = |range: Range, level: BidiLevel| {
         let dir = if level.is_ltr() { Dir::LTR } else { Dir::RTL };
         let shaped =
@@ -665,7 +787,8 @@ fn shape<'a>(
     lang: Lang,
     region: Option<Region>,
 ) -> ShapedText<'a> {
-    let size = TextElem::size_in(styles);
+    let size = styles.resolve(TextElem::size);
+    let shift_settings = styles.get(TextElem::shift_settings);
     let mut ctx = ShapingContext {
         engine,
         size,
@@ -674,8 +797,9 @@ fn shape<'a>(
         styles,
         variant: variant(styles),
         features: features(styles),
-        fallback: TextElem::fallback_in(styles),
+        fallback: styles.get(TextElem::fallback),
         dir,
+        shift_settings,
     };
 
     if !text.is_empty() {
@@ -698,9 +822,7 @@ fn shape<'a>(
         region,
         styles,
         variant: ctx.variant,
-        size,
-        width: ctx.glyphs.iter().map(|g| g.x_advance).sum::<Em>().at(size),
-        glyphs: Cow::Owned(ctx.glyphs),
+        glyphs: Glyphs::from_vec(ctx.glyphs),
     }
 }
 
@@ -708,6 +830,10 @@ fn shape<'a>(
 struct ShapingContext<'a, 'v> {
     engine: &'a Engine<'v>,
     glyphs: Vec<ShapedGlyph>,
+    /// Font families that have been used with unlimited coverage.
+    ///
+    /// These font families are considered exhausted and will not be used again,
+    /// even if they are declared again (e.g., during fallback after normal selection).
     used: Vec<Font>,
     styles: StyleChain<'a>,
     size: Abs,
@@ -715,6 +841,7 @@ struct ShapingContext<'a, 'v> {
     features: Vec<rustybuzz::Feature>,
     fallback: bool,
     dir: Dir,
+    shift_settings: Option<ShiftSettings>,
 }
 
 /// Shape text with font fallback using the `families` iterator.
@@ -765,13 +892,16 @@ fn shape_segment<'a>(
         return;
     };
 
-    ctx.used.push(font.clone());
+    // This font has been exhausted and will not be used again.
+    if covers.is_none() {
+        ctx.used.push(font.clone());
+    }
 
     // Fill the buffer with our text.
     let mut buffer = UnicodeBuffer::new();
     buffer.push_str(text);
     buffer.set_language(language(ctx.styles));
-    if let Some(script) = TextElem::script_in(ctx.styles).custom().and_then(|script| {
+    if let Some(script) = ctx.styles.get(TextElem::script).custom().and_then(|script| {
         rustybuzz::Script::from_iso15924_tag(Tag::from_bytes(script.as_bytes()))
     }) {
         buffer.set_script(script)
@@ -789,6 +919,18 @@ fn shape_segment<'a>(
     // text extraction.
     buffer.set_flags(BufferFlags::REMOVE_DEFAULT_IGNORABLES);
 
+    let (script_shift, script_compensation, scale, shift_feature) = ctx
+        .shift_settings
+        .map_or((Em::zero(), Em::zero(), Em::one(), None), |settings| {
+            determine_shift(text, &font, settings)
+        });
+
+    let has_shift_feature = shift_feature.is_some();
+    if let Some(feat) = shift_feature {
+        // Temporarily push the feature.
+        ctx.features.push(feat)
+    }
+
     // Prepare the shape plan. This plan depends on direction, script, language,
     // and features, but is independent from the text and can thus be memoized.
     let plan = create_shape_plan(
@@ -799,6 +941,10 @@ fn shape_segment<'a>(
         &ctx.features,
     );
 
+    if has_shift_feature {
+        ctx.features.pop();
+    }
+
     // Shape!
     let buffer = rustybuzz::shape_with_plan(font.rusty(), &plan, buffer);
     let infos = buffer.glyph_infos();
@@ -812,7 +958,7 @@ fn shape_segment<'a>(
             .nth(1)
             .map(|(i, _)| offset + i)
             .unwrap_or(text.len());
-        covers.map_or(true, |cov| cov.is_match(&text[offset..end]))
+        covers.is_none_or(|cov| cov.is_match(&text[offset..end]))
     };
 
     // Collect the shaped glyphs, doing fallback and shaping parts again with
@@ -824,12 +970,42 @@ fn shape_segment<'a>(
 
         // Add the glyph to the shaped output.
         if info.glyph_id != 0 && is_covered(cluster) {
-            // Determine the text range of the glyph.
+            // Assume we have the following sequence of (glyph_id, cluster):
+            // [(120, 0), (80, 0), (3, 3), (755, 4), (69, 4), (424, 13),
+            //  (63, 13), (193, 25), (80, 25), (3, 31)
+            //
+            // We then want the sequence of (glyph_id, text_range) to look as follows:
+            // [(120, 0..3), (80, 0..3), (3, 3..4), (755, 4..13), (69, 4..13),
+            //  (424, 13..25), (63, 13..25), (193, 25..31), (80, 25..31), (3, 31..x)]
+            //
+            // Each glyph in the same cluster should be assigned the full text
+            // range. This is necessary because only this way krilla can
+            // properly assign `ActualText` attributes in complex shaping
+            // scenarios.
+
+            // The start of the glyph's text range.
             let start = base + cluster;
-            let end = base
-                + if ltr { i.checked_add(1) } else { i.checked_sub(1) }
-                    .and_then(|last| infos.get(last))
-                    .map_or(text.len(), |info| info.cluster as usize);
+
+            // Determine the end of the glyph's text range.
+            let mut k = i;
+            let step: isize = if ltr { 1 } else { -1 };
+            let end = loop {
+                // If we've reached the end of the glyphs, the `end` of the
+                // range should be the end of the full text.
+                let Some((next, next_info)) = k
+                    .checked_add_signed(step)
+                    .and_then(|n| infos.get(n).map(|info| (n, info)))
+                else {
+                    break base + text.len();
+                };
+
+                // If the cluster doesn't match anymore, we've reached the end.
+                if next_info.cluster != info.cluster {
+                    break base + next_info.cluster as usize;
+                }
+
+                k = next;
+            };
 
             let c = text[cluster..].chars().next().unwrap();
             let script = c.script();
@@ -839,8 +1015,9 @@ fn shape_segment<'a>(
                 glyph_id: info.glyph_id as u16,
                 // TODO: Don't ignore y_advance.
                 x_advance,
-                x_offset: font.to_em(pos[i].x_offset),
-                y_offset: font.to_em(pos[i].y_offset),
+                x_offset: font.to_em(pos[i].x_offset) + script_compensation,
+                y_offset: font.to_em(pos[i].y_offset) + script_shift,
+                size: scale.at(ctx.size),
                 adjustability: Adjustability::default(),
                 range: start..end,
                 safe_to_break: !info.unsafe_to_break(),
@@ -902,9 +1079,67 @@ fn shape_segment<'a>(
     ctx.used.pop();
 }
 
+/// Returns a `(script_shift, script_compensation, scale, feature)` quadruplet
+/// describing how to produce scripts.
+///
+/// Those values determine how the rendered text should be transformed to
+/// display sub-/super-scripts. If the OpenType feature can be used, the
+/// rendered text should not be transformed in any way, and so those values are
+/// neutral (`(0, 0, 1, None)`). If scripts should be synthesized, those values
+/// determine how to transform the rendered text to display scripts as expected.
+fn determine_shift(
+    text: &str,
+    font: &Font,
+    settings: ShiftSettings,
+) -> (Em, Em, Em, Option<Feature>) {
+    settings
+        .typographic
+        .then(|| {
+            // If typographic scripts are enabled (i.e., we want to use the
+            // OpenType feature instead of synthesizing if possible), we add
+            // "subs"/"sups" to the feature list if supported by the font.
+            // In case of a problem, we just early exit
+            let gsub = font.rusty().tables().gsub?;
+            let subtable_index =
+                gsub.features.find(settings.kind.feature())?.lookup_indices.get(0)?;
+            let coverage = gsub
+                .lookups
+                .get(subtable_index)?
+                .subtables
+                .get::<SubstitutionSubtable>(0)?
+                .coverage();
+            text.chars()
+                .all(|c| {
+                    font.rusty().glyph_index(c).is_some_and(|i| coverage.contains(i))
+                })
+                .then(|| {
+                    // If we can use the OpenType feature, we can keep the text
+                    // as is.
+                    (
+                        Em::zero(),
+                        Em::zero(),
+                        Em::one(),
+                        Some(Feature::new(settings.kind.feature(), 1, ..)),
+                    )
+                })
+        })
+        // Reunite the cases where `typographic` is `false` or where using the
+        // OpenType feature would not work.
+        .flatten()
+        .unwrap_or_else(|| {
+            let script_metrics = settings.kind.read_metrics(font.metrics());
+            (
+                settings.shift.unwrap_or(script_metrics.vertical_offset),
+                script_metrics.horizontal_offset,
+                settings.size.unwrap_or(script_metrics.height),
+                None,
+            )
+        })
+}
+
 /// Create a shape plan.
 #[comemo::memoize]
-fn create_shape_plan(
+pub fn create_shape_plan(
     font: &Font,
     direction: rustybuzz::Direction,
     script: rustybuzz::Script,
@@ -922,7 +1157,7 @@ fn create_shape_plan(
 
 /// Shape the text with tofus from the given font.
 fn shape_tofus(ctx: &mut ShapingContext, base: usize, text: &str, font: Font) {
-    let x_advance = font.advance(0).unwrap_or_default();
+    let x_advance = font.x_advance(0).unwrap_or_default();
     let add_glyph = |(cluster, c): (usize, char)| {
         let start = base + cluster;
         let end = start + c.len_utf8();
@@ -933,6 +1168,7 @@ fn shape_tofus(ctx: &mut ShapingContext, base: usize, text: &str, font: Font) {
             x_advance,
             x_offset: Em::zero(),
             y_offset: Em::zero(),
+            size: ctx.size,
             adjustability: Adjustability::default(),
             range: start..end,
             safe_to_break: true,
@@ -955,9 +1191,11 @@ fn shape_tofus(ctx: &mut ShapingContext, base: usize, text: &str, font: Font) {
 
 /// Apply tracking and spacing to the shaped glyphs.
 fn track_and_space(ctx: &mut ShapingContext) {
-    let tracking = Em::from_length(TextElem::tracking_in(ctx.styles), ctx.size);
-    let spacing =
-        TextElem::spacing_in(ctx.styles).map(|abs| Em::from_length(abs, ctx.size));
+    let tracking = Em::from_abs(ctx.styles.resolve(TextElem::tracking), ctx.size);
+    let spacing = ctx
+        .styles
+        .resolve(TextElem::spacing)
+        .map(|abs| Em::from_abs(abs, ctx.size));
 
     let mut glyphs = ctx.glyphs.iter_mut().peekable();
     while let Some(glyph) = glyphs.next() {
@@ -983,9 +1221,18 @@ fn track_and_space(ctx: &mut ShapingContext) {
 /// and CJK punctuation adjustments according to Chinese Layout Requirements.
 fn calculate_adjustability(ctx: &mut ShapingContext, lang: Lang, region: Option<Region>) {
     let style = cjk_punct_style(lang, region);
+    let limits = ctx.styles.get(ParElem::justification_limits);
+    let font_size = ctx.size;
 
-    for glyph in &mut ctx.glyphs {
-        glyph.adjustability = glyph.base_adjustability(style);
+    let mut glyphs = ctx.glyphs.iter_mut().peekable();
+    while let Some(glyph) = glyphs.next() {
+        // Do not apply adjustability to a glyph if there is another one in the
+        // same cluster.
+        let stretchable =
+            glyphs.peek().is_none_or(|next| glyph.range.start != next.range.start);
+
+        glyph.adjustability =
+            glyph.base_adjustability(style, &limits, font_size, stretchable);
     }
 
     let mut glyphs = ctx.glyphs.iter_mut().peekable();
@@ -1016,18 +1263,7 @@ fn calculate_adjustability(ctx: &mut ShapingContext, lang: Lang, region: Option<
 fn nbsp_delta(font: &Font) -> Option<Em> {
     let space = font.ttf().glyph_index(' ')?.0;
     let nbsp = font.ttf().glyph_index('\u{00A0}')?.0;
-    Some(font.advance(nbsp)? - font.advance(space)?)
-}
-
-/// Process the language and region of a style chain into a
-/// rustybuzz-compatible BCP 47 language.
-fn language(styles: StyleChain) -> rustybuzz::Language {
-    let mut bcp: EcoString = TextElem::lang_in(styles).as_str().into();
-    if let Some(region) = TextElem::region_in(styles) {
-        bcp.push('-');
-        bcp.push_str(region.as_str());
-    }
-    rustybuzz::Language::from_str(&bcp).unwrap()
+    Some(font.x_advance(nbsp)? - font.x_advance(space)?)
 }
 
 /// Returns true if all glyphs in `glyphs` have ranges within the range `range`.
@@ -1077,7 +1313,7 @@ pub const END_PUNCT_PAT: &[char] = &[
     '〗', '〕', '］', '｝', '？', '！',
 ];
 
-#[derive(Debug, Clone, Copy, PartialEq, Eq)]
+#[derive(Debug, Copy, Clone, Eq, PartialEq)]
 pub enum CjkPunctStyle {
     /// Standard GB/T 15834-2011, used mostly in mainland China.
     Gb,
