@@ -5,9 +5,9 @@ use std::cmp::Ordering;
 use ecow::eco_format;
 use typst_utils::Numeric;
 
-use crate::diag::{bail, DeprecationSink, HintedStrResult, StrResult};
+use crate::diag::{HintedStrResult, StrResult, bail};
 use crate::foundations::{
-    format_str, Datetime, IntoValue, Regex, Repr, SymbolElem, Value,
+    Datetime, IntoValue, Regex, Repr, SymbolElem, Value, format_str,
 };
 use crate::layout::{Alignment, Length, Rel};
 use crate::text::TextElem;
@@ -21,7 +21,7 @@ macro_rules! mismatch {
 }
 
 /// Join a value with another value.
-pub fn join(lhs: Value, rhs: Value, sink: &mut dyn DeprecationSink) -> StrResult<Value> {
+pub fn join(lhs: Value, rhs: Value) -> StrResult<Value> {
     use Value::*;
     Ok(match (lhs, rhs) {
         (a, None) => a,
@@ -39,17 +39,6 @@ pub fn join(lhs: Value, rhs: Value, sink: &mut dyn DeprecationSink) -> StrResult
         (Array(a), Array(b)) => Array(a + b),
         (Dict(a), Dict(b)) => Dict(a + b),
         (Args(a), Args(b)) => Args(a + b),
-
-        // Type compatibility.
-        (Type(a), Str(b)) => {
-            warn_type_str_join(sink);
-            Str(format_str!("{a}{b}"))
-        }
-        (Str(a), Type(b)) => {
-            warn_type_str_join(sink);
-            Str(format_str!("{a}{b}"))
-        }
-
         (a, b) => mismatch!("cannot join {} with {}", a, b),
     })
 }
@@ -99,11 +88,7 @@ pub fn neg(value: Value) -> HintedStrResult<Value> {
 }
 
 /// Compute the sum of two values.
-pub fn add(
-    lhs: Value,
-    rhs: Value,
-    sink: &mut dyn DeprecationSink,
-) -> HintedStrResult<Value> {
+pub fn add(lhs: Value, rhs: Value) -> HintedStrResult<Value> {
     use Value::*;
     Ok(match (lhs, rhs) {
         (a, None) => a,
@@ -171,16 +156,6 @@ pub fn add(
         (Datetime(a), Duration(b)) => Datetime(a + b),
         (Duration(a), Datetime(b)) => Datetime(b + a),
 
-        // Type compatibility.
-        (Type(a), Str(b)) => {
-            warn_type_str_add(sink);
-            Str(format_str!("{a}{b}"))
-        }
-        (Str(a), Type(b)) => {
-            warn_type_str_add(sink);
-            Str(format_str!("{a}{b}"))
-        }
-
         (Dyn(a), Dyn(b)) => {
             // Alignments can be summed.
             if let (Some(&a), Some(&b)) =
@@ -419,21 +394,13 @@ pub fn or(lhs: Value, rhs: Value) -> HintedStrResult<Value> {
 }
 
 /// Compute whether two values are equal.
-pub fn eq(
-    lhs: Value,
-    rhs: Value,
-    sink: &mut dyn DeprecationSink,
-) -> HintedStrResult<Value> {
-    Ok(Value::Bool(equal(&lhs, &rhs, sink)))
+pub fn eq(lhs: Value, rhs: Value) -> HintedStrResult<Value> {
+    Ok(Value::Bool(equal(&lhs, &rhs)))
 }
 
 /// Compute whether two values are unequal.
-pub fn neq(
-    lhs: Value,
-    rhs: Value,
-    sink: &mut dyn DeprecationSink,
-) -> HintedStrResult<Value> {
-    Ok(Value::Bool(!equal(&lhs, &rhs, sink)))
+pub fn neq(lhs: Value, rhs: Value) -> HintedStrResult<Value> {
+    Ok(Value::Bool(!equal(&lhs, &rhs)))
 }
 
 macro_rules! comparison {
@@ -452,7 +419,7 @@ comparison!(gt, ">", Ordering::Greater);
 comparison!(geq, ">=", Ordering::Greater | Ordering::Equal);
 
 /// Determine whether two values are equal.
-pub fn equal(lhs: &Value, rhs: &Value, sink: &mut dyn DeprecationSink) -> bool {
+pub fn equal(lhs: &Value, rhs: &Value) -> bool {
     use Value::*;
     match (lhs, rhs) {
         // Compare reflexively.
@@ -496,12 +463,6 @@ pub fn equal(lhs: &Value, rhs: &Value, sink: &mut dyn DeprecationSink) -> bool {
             rat == rel.rel && rel.abs.is_zero()
         }
 
-        // Type compatibility.
-        (Type(ty), Str(str)) | (Str(str), Type(ty)) => {
-            warn_type_str_equal(sink, str);
-            ty.compat_name() == str.as_str()
-        }
-
         _ => false,
     }
 }
@@ -573,12 +534,8 @@ fn try_cmp_arrays(a: &[Value], b: &[Value]) -> StrResult<Ordering> {
 }
 
 /// Test whether one value is "in" another one.
-pub fn in_(
-    lhs: Value,
-    rhs: Value,
-    sink: &mut dyn DeprecationSink,
-) -> HintedStrResult<Value> {
-    if let Some(b) = contains(&lhs, &rhs, sink) {
+pub fn in_(lhs: Value, rhs: Value) -> HintedStrResult<Value> {
+    if let Some(b) = contains(&lhs, &rhs) {
         Ok(Value::Bool(b))
     } else {
         mismatch!("cannot apply 'in' to {} and {}", lhs, rhs)
@@ -586,12 +543,8 @@ pub fn in_(
 }
 
 /// Test whether one value is "not in" another one.
-pub fn not_in(
-    lhs: Value,
-    rhs: Value,
-    sink: &mut dyn DeprecationSink,
-) -> HintedStrResult<Value> {
-    if let Some(b) = contains(&lhs, &rhs, sink) {
+pub fn not_in(lhs: Value, rhs: Value) -> HintedStrResult<Value> {
+    if let Some(b) = contains(&lhs, &rhs) {
         Ok(Value::Bool(!b))
     } else {
         mismatch!("cannot apply 'not in' to {} and {}", lhs, rhs)
@@ -599,27 +552,14 @@ pub fn not_in(
 }
 
 /// Test for containment.
-pub fn contains(
-    lhs: &Value,
-    rhs: &Value,
-    sink: &mut dyn DeprecationSink,
-) -> Option<bool> {
+pub fn contains(lhs: &Value, rhs: &Value) -> Option<bool> {
     use Value::*;
     match (lhs, rhs) {
         (Str(a), Str(b)) => Some(b.as_str().contains(a.as_str())),
         (Dyn(a), Str(b)) => a.downcast::<Regex>().map(|regex| regex.is_match(b)),
         (Str(a), Dict(b)) => Some(b.contains(a)),
-        (a, Array(b)) => Some(b.contains_impl(a, sink)),
-
-        // Type compatibility.
-        (Type(a), Str(b)) => {
-            warn_type_in_str(sink);
-            Some(b.as_str().contains(a.compat_name()))
-        }
-        (Type(a), Dict(b)) => {
-            warn_type_in_dict(sink);
-            Some(b.contains(a.compat_name()))
-        }
+        (Str(a), Module(b)) => Some(b.scope().get(a).is_some()),
+        (a, Array(b)) => Some(b.contains(a.clone())),
 
         _ => Option::None,
     }
@@ -629,90 +569,3 @@ pub fn contains(
 fn too_large() -> &'static str {
     "value is too large"
 }
-
-#[cold]
-fn warn_type_str_add(sink: &mut dyn DeprecationSink) {
-    sink.emit_with_hints(
-        "adding strings and types is deprecated",
-        &["convert the type to a string with `str` first"],
-    );
-}
-
-#[cold]
-fn warn_type_str_join(sink: &mut dyn DeprecationSink) {
-    sink.emit_with_hints(
-        "joining strings and types is deprecated",
-        &["convert the type to a string with `str` first"],
-    );
-}
-
-#[cold]
-fn warn_type_str_equal(sink: &mut dyn DeprecationSink, s: &str) {
-    // Only warn if `s` looks like a type name to prevent false positives.
-    if is_compat_type_name(s) {
-        sink.emit_with_hints(
-            "comparing strings with types is deprecated",
-            &[
-                "compare with the literal type instead",
-                "this comparison will always return `false` in future Typst releases",
-            ],
-        );
-    }
-}
-
-#[cold]
-fn warn_type_in_str(sink: &mut dyn DeprecationSink) {
-    sink.emit_with_hints(
-        "checking whether a type is contained in a string is deprecated",
-        &["this compatibility behavior only exists because `type` used to return a string"],
-    );
-}
-
-#[cold]
-fn warn_type_in_dict(sink: &mut dyn DeprecationSink) {
-    sink.emit_with_hints(
-        "checking whether a type is contained in a dictionary is deprecated",
-        &["this compatibility behavior only exists because `type` used to return a string"],
-    );
-}
-
-fn is_compat_type_name(s: &str) -> bool {
-    matches!(
-        s,
-        "boolean"
-            | "alignment"
-            | "angle"
-            | "arguments"
-            | "array"
-            | "bytes"
-            | "color"
-            | "content"
-            | "counter"
-            | "datetime"
-            | "decimal"
-            | "dictionary"
-            | "direction"
-            | "duration"
-            | "float"
-            | "fraction"
-            | "function"
-            | "gradient"
-            | "integer"
-            | "label"
-            | "length"
-            | "location"
-            | "module"
-            | "pattern"
-            | "ratio"
-            | "regex"
-            | "relative length"
-            | "selector"
-            | "state"
-            | "string"
-            | "stroke"
-            | "symbol"
-            | "tiling"
-            | "type"
-            | "version"
-    )
-}
